local function assemblerpipepictures()
  return {
    north = {
      filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-k-pipe-N.png",
      priority = "extra-high",
      width = 71,
      height = 38,
      shift = util.by_pixel(2.25, 13.5),
      scale = 0.5,
    },
    east = {
      filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-k-pipe-E.png",
      priority = "extra-high",
      width = 42,
      height = 76,
      shift = util.by_pixel(-24.5, 1),
      scale = 0.5,
    },
    south = {
      filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-k-pipe-S.png",
      priority = "extra-high",
      width = 88,
      height = 61,
      shift = util.by_pixel(0, -31.25),
      scale = 0.5,
    },
    west = {
      filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-k-pipe-W.png",
      priority = "extra-high",
      width = 39,
      height = 73,
      shift = util.by_pixel(25.75, 1.25),
      scale = 0.5,
    },
  }
end

local hit_effects = require("__base__/prototypes/entity/hit-effects")
local sounds = require("__base__/prototypes/entity/sounds")
require("__Ultracube__/prototypes/entities/lib/module_effects")

data:extend({
  {
    type = "assembling-machine",
    name = "cube-fabricator",
    icon = "__Krastorio2Assets__/icons/entities/advanced-assembling-machine.png",
    icon_size = 64,
    icon_mipmaps = 4,
    flags = {"placeable-neutral", "placeable-player", "player-creation"},
    minable = {mining_time = 0.5, result = "cube-fabricator"},
    max_health = 800,
    corpse = "cube-medium-random-pipes-remnant",
    dying_explosion = "big-explosion",
    collision_box = {{-1.75, -1.75}, {1.75, 1.75}},
    selection_box = {{-2, -2}, {2, 2}},
    damaged_trigger_effect = hit_effects.entity(),
    fluid_boxes = {
      {
        production_type = "input",
        pipe_picture = assemblerpipepictures(),
        pipe_covers = pipecoverspictures(),
        volume = 1000,
        pipe_connections = {{flow_direction = "input", direction = defines.direction.north, position = {-0.5, -1.5}}},
        secondary_draw_orders = {north = -1},
      },
      {
        production_type = "output",
        pipe_picture = assemblerpipepictures(),
        pipe_covers = pipecoverspictures(),
        volume = 1000,
        pipe_connections = {{flow_direction = "output", direction = defines.direction.south, position = {0.5, 1.5}}},
        secondary_draw_orders = {north = -1},
      },
    },
    fluid_boxes_off_when_no_fluid_recipe = true,

    graphics_set = {
      animation = {
        layers = {
          {
            filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine.png",
            priority = "high",
            width = 320,
            height = 320,
            frame_count = 1,
            repeat_count = 32,
            animation_speed = 1,
            shift = {0, 0},
            scale = 0.4,
          },
          {
            filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-w1.png",
            priority = "high",
            width = 128,
            height = 144,
            shift = {-1.02 * 0.8, 0.29 * 0.8},
            frame_count = 32,
            line_length = 8,
            animation_speed = 0.5,
            scale = 0.4,
          },
          {
            filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-steam.png",
            priority = "high",
            width = 80,
            height = 81,
            shift = {-1.2 * 0.8, -2.1 * 0.8},
            frame_count = 32,
            line_length = 8,
            animation_speed = 1.5,
            scale = 0.4,
          },
          {
            filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-sh.png",
            priority = "high",
            width = 346,
            height = 302,
            shift = {0.32 * 0.8, 0.12 * 0.8},
            frame_count = 1,
            repeat_count = 32,
            animation_speed = 0.5,
            draw_as_shadow = true,
            scale = 0.4,
          },
          {
            filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-w2.png",
            priority = "high",
            width = 37,
            height = 25,
            frame_count = 8,
            line_length = 4,
            repeat_count = 4,
            animation_speed = 0.5,
            shift = {0.17 * 0.8, -1.445 * 0.8},
            scale = 0.4,
          },
          {
            filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-w3.png",
            priority = "high",
            width = 23,
            height = 15,
            frame_count = 8,
            line_length = 4,
            repeat_count = 4,
            animation_speed = 0.5,
            shift = {0.93 * 0.8, -2.05 * 0.8},
            scale = 0.4,
          },
          {
            filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-w3.png",
            priority = "high",
            width = 23,
            height = 15,
            frame_count = 8,
            line_length = 4,
            repeat_count = 4,
            animation_speed = 0.5,
            shift = {0.868 * 0.8, -0.082 * 0.8},
            scale = 0.4,
          },
          {
            filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-w3.png",
            priority = "high",
            width = 23,
            height = 15,
            frame_count = 8,
            line_length = 4,
            repeat_count = 4,
            animation_speed = 0.5,
            shift = {0.868 * 0.8, 0.552 * 0.8},
            scale = 0.4,
          },
        },
      },
    },
    crafting_categories = {"cube-fabricator", "cube-fabricator-handcraft"},
    vehicle_impact_sound = sounds.generic_impact,
    working_sound = {
      sound = {
        {
          filename = "__Krastorio2Assets__/sounds/buildings/advanced-assembling-machine.ogg",
          volume = 0.8,
        },
      },
      idle_sound = {filename = "__base__/sound/idle1.ogg", volume = 0.3},
      apparent_volume = 1.5,
      max_sounds_per_type = 3,
      fade_in_ticks = 10,
      fade_out_ticks = 30,
    },
    idle_sound = {filename = "__base__/sound/idle1.ogg"},
    crafting_speed = 1,
    energy_source = {
      type = "electric",
      usage_priority = "secondary-input",
      emissions_per_minute = {},
      drain = "10kW",
    },
    energy_usage = "150kW",

    water_reflection = {
      pictures = {
        filename = "__Krastorio2Assets__/entities/advanced-assembling-machine/advanced-assembling-machine-reflection.png",
        priority = "extra-high",
        width = 70,
        height = 50,
        shift = util.by_pixel(0, 40),
        variation_count = 1,
        scale = 4,
      },
      rotate = false,
      orientation_to_variation = false,
    },

    module_slots = 4,
    icon_draw_specification = {scale = 1.25, shift = {0, -0.3}},
    icons_positioning = {{
      inventory_index = defines.inventory.assembling_machine_modules,
      shift = {0, 0.7},
      scale = 0.625,
    }},
    allowed_effects = module_effects.all,
    open_sound = {filename = "__base__/sound/machine-open.ogg", volume = 0.75},
    close_sound = {filename = "__base__/sound/machine-close.ogg", volume = 0.75},
  },
})
