local hit_effects = require("__base__/prototypes/entity/hit-effects")
local sounds = require("__base__/prototypes/entity/sounds")
require("__Ultracube__/prototypes/entities/lib/module_effects")
require("__Ultracube__/scripts/lib")

local function stygian_smoke()
  local smoke = {}
  for i = 2, 4 do
    smoke[#smoke+1] = {
      name = "cube-stygian-energy-lab-smoke",
      position = from_polar(1, 2 * math.pi * i / 4),
      deviation = {0.1, 0.1},
      frequency = 8,
      starting_vertical_speed = 1.0 / 16,
      starting_frame_deviation = i * 10,
      starting_frame = i * 5,
    }
  end
  return smoke
end

data:extend({
  {
    type = "furnace",
    name = "cube-mystery-furnace",
    icon = "__base__/graphics/icons/steel-furnace.png",
    icon_size = 64, icon_mipmaps = 4,
    flags = {"placeable-neutral", "placeable-player", "player-creation"},
    show_recipe_icon_on_map = false,
    minable = {mining_time = 0.2, result = "cube-mystery-furnace"},
    max_health = 500,
    corpse = "steel-furnace-remnants",
    dying_explosion = "steel-furnace-explosion",
    vehicle_impact_sound = sounds.generic_impact,
    open_sound = sounds.machine_open,
    close_sound = sounds.machine_close,
    working_sound = {
      sound = {
        {
          filename = "__base__/sound/steel-furnace.ogg",
          volume = 0.46
        }
      },
      max_sounds_per_type = 4,
      audible_distance_modifier = 0.37,
      fade_in_ticks = 4,
      fade_out_ticks = 20
    },
    collision_box = {{-0.7, -0.7}, {0.7, 0.7}},
    selection_box = {{-0.8, -1}, {0.8, 1}},
    damaged_trigger_effect = hit_effects.entity(),
    crafting_categories = {"cube-mystery-furnace"},
    result_inventory_size = 2,
    energy_usage = "24MW",
    crafting_speed = 1,
    source_inventory_size = 1,
    energy_source = {
      type = "burner",
      fuel_categories = {"cube-haunted-energy"},
      effectivity = 1,
      emissions_per_minute = {},
      fuel_inventory_size = 1,
      light_flicker = {
        color = {r = 0.4, g = 0.8, b = 1},
        minimum_intensity = 0.6,
        maximum_intensity = 0.95,
      },
      smoke = stygian_smoke(),
    },
    module_slots = 0,
    allowed_effects = module_effects.none,
    graphics_set = {
      animation = {
        layers = {
          {
            filename = "__base__/graphics/entity/steel-furnace/steel-furnace.png",
            priority = "high",
            width = 171,
            height = 174,
            frame_count = 1,
            shift = util.by_pixel(-1.25, 2),
            scale = 0.5,
          },
          {
            filename = "__base__/graphics/entity/steel-furnace/steel-furnace-shadow.png",
            priority = "high",
            width = 277,
            height = 85,
            frame_count = 1,
            draw_as_shadow = true,
            shift = util.by_pixel(39.25, 11.25),
            scale = 0.5,
          }
        }
      },
      working_visualisations = {
        {
          draw_as_light = true,
          fadeout = true,
          effect = "flicker",
          animation = {
            filename = "__base__/graphics/entity/steel-furnace/steel-furnace-fire.png",
            priority = "high",
            line_length = 8,
            width = 57,
            height = 81,
            frame_count = 48,
            direction_count = 1,
            shift = util.by_pixel(-0.75, 5.75),
            scale = 0.5,
          },
        },
        {
          fadeout = true,
          draw_as_light = true,
          effect = "flicker",
          animation = {
            filename = "__base__/graphics/entity/steel-furnace/steel-furnace-glow.png",
            priority = "high",
            width = 60,
            height = 43,
            frame_count = 1,
            shift = {0.03125, 0.640625},
            blend_mode = "additive"
          }
        },
        {
          fadeout = true,
          draw_as_light = true,
          effect = "flicker",
          animation = {
            filename = "__base__/graphics/entity/steel-furnace/steel-furnace-working.png",
            priority = "high",
            line_length = 1,
            width = 128,
            height = 150,
            frame_count = 1,
            direction_count = 1,
            shift = util.by_pixel(0, -5),
            blend_mode = "additive",
            scale = 0.5,
          }
        },
        {
          draw_as_light = true,
          draw_as_sprite = false,
          fadeout = true,
          effect = "flicker",
          animation = {
            filename = "__base__/graphics/entity/steel-furnace/steel-furnace-ground-light.png",
            priority = "high",
            line_length = 1,
            draw_as_sprite = false,
            width = 152,
            height = 126,
            frame_count = 1,
            direction_count = 1,
            shift = util.by_pixel(1, 48),
            blend_mode = "additive",
            scale = 0.5,
          },
        },
      },
    },
    fast_replaceable_group = "furnace",
    water_reflection = {
      pictures = {
        filename = "__base__/graphics/entity/steel-furnace/steel-furnace-reflection.png",
        priority = "extra-high",
        width = 20,
        height = 24,
        shift = util.by_pixel(0, 45),
        variation_count = 1,
        scale = 5
      },
      rotate = false,
      orientation_to_variation = false
    }
  },
})