require("__Ultracube__/scripts/lib")

local vanilla_fluids = make_set({"water", "sulfuric-acid"})
for name, fluid in pairs(data.raw.fluid) do
  if not vanilla_fluids[name] then
    fluid.auto_barrel = false
  end
end
data.raw.fluid["sulfuric-acid"].barrelling_unlock_tech = "cube-sulfur-processing"

data:extend({
  {
    type = "fluid",
    name = "cube-gelatinous-tar",
    icon = "__Ultracube__/assets/icons/tar.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.1, g=0.1, b=0.15},
    flow_color = {r=0.15, g=0.15, b=0.20},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-tar-furnace",
    order = "cube-0[a-tar]",
    barrelling_unlock_tech = "cube-tar-processing",
  },
  {
    type = "fluid",
    name = "cube-heavy-tar",
    icon = "__Krastorio2Assets__/icons/fluids/heavy-oil.png",
    icon_size = 64, icon_mipmaps = 4,
    -- TODO: check colour matches icon.
    base_color = {r=0.5, g=0.04, b=0},
    flow_color = {r=0.85, g=0.6, b=0.3},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-tar-input",
    order = "cube-0[tar]-a[heavy]",
    barrelling_unlock_tech = "cube-tar-processing",
  },
  {
    type = "fluid",
    name = "cube-light-tar",
    icon = "__Krastorio2Assets__/icons/fluids/light-oil.png",
    icon_size = 64, icon_mipmaps = 4,
    -- TODO: check colour matches icon.
    base_color = {r=0.57, g=0.33, b=0},
    flow_color = {r=1, g=0.73, b=0.07},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-tar-input",
    order = "cube-0[tar]-b[light]",
    barrelling_unlock_tech = "cube-tar-processing",
  },
  {
    type = "fluid",
    name = "cube-unstable-gas",
    icon = "__Krastorio2Assets__/icons/fluids/petroleum-gas.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.3, g=0.1, b=0.3},
    flow_color = {r=0.8, g=0.8, b=0.8},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-tar-output",
    order = "cube-a[tar]-a[unstable-gas]",
    auto_barrel = false,
  },
  {
    type = "fluid",
    name = "cube-elbow-grease",
    icon = "__Krastorio2Assets__/icons/fluids/lubricant.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.15, g=0.32, b=0.03},
    flow_color = {r=0.43, g=0.75, b=0.31},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-tar-output",
    order = "cube-a[tar]-c[lubricant]",
    barrelling_unlock_tech = "cube-elbow-grease",
  },
  {
    type = "fluid",
    name = "cube-ethanol",
    icon = "__Ultracube__/assets/icons/ethanol.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.5, g=0.5, b=0.5, a=0.5},
    flow_color = {r=0.6, g=0.6, b=0.6, a=0.5},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-distillation",
    order = "cube-2[ethanol]",
    barrelling_unlock_tech = "cube-distillation",
  },
  {
    type = "fluid",
    name = "cube-matter-replication-gel",
    icon = "__Krastorio2Assets__/icons/fluids/hydrogen.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.8, g=0.8, b=0.8},
    flow_color = {r=0.9, g=0.9, b=0.9},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-synthesis-basics",
    order = "cube-1[0-cube-matter-replication-gel]",
    auto_barrel = false,
  },
  {
    type = "fluid",
    name = "cube-cerebral-substrate",
    icon = "__Krastorio2Assets__/icons/fluids/matter.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.15, g=0.8, b=0.8},
    flow_color = {r=0.2, g=0.9, b=0.9},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-synthesis-sentience",
    order = "cube-0[a-cube-cerebral-substrate]",
    auto_barrel = false,
  },
  {
    type = "fluid",
    name = "cube-joy",
    icon = "__Ultracube__/assets/icons/joy.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.85, g=0.1, b=0.4},
    flow_color = {r=0.95, g=0.15, b=0.5},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-synthesis-fluids",
    order = "cube-b[matter]-c[joy]",
    auto_barrel = false,
  },
  {
    type = "fluid",
    name = "cube-despair",
    icon = "__Ultracube__/assets/icons/despair.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.35, g=0.15, b=0.7},
    flow_color = {r=0.4, g=0.2, b=0.75},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-synthesis-fluids",
    order = "cube-b[matter]-d[despair]",
    auto_barrel = false,
  },
  {
    type = "fluid",
    name = "cube-mineral-water",
    icon = "__Krastorio2Assets__/icons/fluids/mineral-water.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.15, g=0.1, b=0.4},
    flow_color = {r=0.5, g=0.5, b=0.75},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-distillation",
    order = "cube-1[mineral-water]",
    barrelling_unlock_tech = "cube-deep-crystallization",
  },
  {
    type = "fluid",
    name = "cube-deep-solution",
    icon = "__Ultracube__/assets/icons/deep-solution.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.7, g=0.55, b=0.8},
    flow_color = {r=0.75, g=0.6, b=0.85},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-distillation",
    order = "cube-0[deep-solution]",
    barrelling_unlock_tech = "cube-deep-crystallization",
  },
  {
    type = "fluid",
    name = "cube-ectoplasm",
    icon = "__Ultracube__/assets/icons/ectoplasm.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.175, g=0.30, b=0.25, a=0.65},
    flow_color = {r=0.65, g=0.9, b=0.8},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-distillation",
    order = "cube-x[ectoplasm]",
    auto_barrel = false,
  },
  {
    type = "fluid",
    name = "cube-radioactive-sludge",
    icon = "__Ultracube__/assets/icons/radioactive-sludge.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.02, g=0.18, b=0.03},
    flow_color = {r=0.05, g=0.35, b=0.05},
    default_temperature = 25,
    subgroup = "cube-energy-fluids",
    order = "cube-0[sludge]",
    barrelling_unlock_tech = "cube-kovarex-enrichment-process",
  },
  {
    type = "fluid",
    name = "cube-beacon-juice",
    icon = "__Ultracube__/assets/icons/beacon-juice.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.0, g=0.975, b=0.65},
    flow_color = {r=0.15, g=0.975, b=0.925},
    default_temperature = 25,
    fuel_value = "400kJ",
    subgroup = "cube-distillation",
    order = "cube-y[beacon-juice]",
    auto_barrel = false,
  },
  {
    type = "fluid",
    name = "cube-ultradense-sap",
    icon = "__Ultracube__/assets/icons/ultradense-sap.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.075, g=0.25, b=0.35},
    flow_color = {r=0.7, g=1, b=1},
    default_temperature = 25,
    heat_capacity = "0.1kJ",
    subgroup = "cube-synthesis-forbidden",
    order = "cube-3[step-2a]",
    auto_barrel = false,
  },
  {
    type = "fluid",
    name = "cube-void-wind",
    icon = "__Ultracube__/assets/icons/void-wind.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.0, g=0.0, b=0.0, a=1},
    flow_color = {r=0.1, g=0.1, b=0.1, a=0.85},
    default_temperature = 25,
    max_temperature = 425,
    heat_capacity = "0.2kJ",
    subgroup = "cube-energy-fluids",
    order = "cube-a[void-wind]",
    auto_barrel = false,
  },
  {
    type = "fluid",
    name = "cube-ionized-annihilation-stream",
    icon = "__Ultracube__/assets/icons/ionized-annihilation-stream.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.5, g=0.7, b=0.8, a=1},
    flow_color = {r=0.75, g=0.85, b=0.9, a=1},
    default_temperature = 25,
    fuel_value = "250kJ",
    subgroup = "cube-energy-fluids",
    order = "cube-c[ion-stream]",
    auto_barrel = false,
  },
  {
    type = "fluid",
    name = "cube-gamma-waste-stream",
    icon = "__Ultracube__/assets/icons/gamma-waste-stream.png",
    icon_size = 64, icon_mipmaps = 4,
    base_color = {r=0.1, g=0.075, b=0.25, a=1},
    flow_color = {r=0.45, g=0.1, b=0.2, a=1},
    default_temperature = 25,
    subgroup = "cube-energy-fluids",
    order = "cube-d[gamma-stream]",
    auto_barrel = false,
  },
})
